/*
 * Copyright (c) 2011, 2012 events-on-fire Team
 * 
 * This file is part of Events-On-Fire (http://code.google.com/p/events-on-fire), licensed under the terms of the MIT
 * License (MIT).
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
 * Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.google.code.eventsonfire;

import com.google.code.eventsonfire.error.ErrorHandler;

/**
 * Holds information about an event handler method.
 * 
 * @author Manfred HANTSCHEL
 */
public interface EventHandlerInfo
{

    /**
     * Invokes the method referenced by this information object if applicable for the type of producer, consumer and
     * event. If an error occurs when invoking the method, the invocationFailed method of the {@link ErrorHandler} is
     * called.
     * 
     * @param producer the producer, mandatory
     * @param consumer the consumer, mandatory
     * @param event the event, mandatory
     * @param tags the tags
     * @return true if invoked (or will be invoked in near future), false otherwise
     */
    boolean invoke(Object producer, Object consumer, Object event, String... tags);

}
